package TankSystem 
  
  connector Sensor 
    Real value;
  end Sensor;
  
  connector Actor 
    Real value;
  end Actor;
  
  connector LiquidFlow 
    Real value;
  end LiquidFlow;
  
  model Tank 
    
    parameter Real area(unit="m2")=1;
    parameter Real flowGain(unit="m2/s")=0.05;
    Real h;
    TankSystem.Sensor Sensor1;
    TankSystem.Actor Actor1;
    TankSystem.LiquidFlow q_in;
    TankSystem.LiquidFlow q_out;
    
  equation 
    der(h)=(q_in.value-q_out.value)/area;
    q_out.value=-flowGain*Actor1.value;
    Sensor1.value=h;
    
  end Tank;
  
  model LiquidSource 
    parameter Real flowrate(unit="m3/s")=0.02;
    TankSystem.LiquidFlow q_out;
    
  equation 
    q_out.value=if time < 150 then flowrate else 3*flowrate;
    
  end LiquidSource;
  
  model PI_Controller 
    
    parameter Real flowGain(unit="m2/s")=0.05;
    parameter Real K=2 "Gain";
    parameter Real T(unit="s")=10 "Time Constant";
    parameter Real ref=0.25 "Reference level";
    Real error "Difference to Reference level";
    Real outCtr "Control signal without limiter";
    Real x "State variable for controller";
    TankSystem.Actor cOut;
    TankSystem.Sensor cIn;
  equation 
    der(x)=error/T;
    outCtr=K*(error+x);
    error=ref-cIn.value;
    cOut.value=outCtr;
    
  end PI_Controller;
  
  model Systemaufbau 
    
  TankSystem.PI_Controller PI_Controller1;
  TankSystem.LiquidSource LiquidSource1;
  TankSystem.Tank Tank1;
    
  equation 
    connect(LiquidSource1.q_out, Tank1.q_in);
    connect(Tank1.Sensor1, PI_Controller1.cIn);
    connect(PI_Controller1.cOut, Tank1.Actor1);
    
  end Systemaufbau;
end TankSystem;
